# -*- coding: UTF-8 -*-

import nes_common.logger as logger
import multiprocessing
import threading
from nes_common.basictaskrunner import BasicTaskRunner


def synchronized(func):
    func.__lock__ = threading.Lock()
        
    def synced_func(*args, **kws):
        with func.__lock__:
            return func(*args, **kws)
    return synced_func


class MprocTaskRunner(BasicTaskRunner):
    """
    Реализация объекта выполняющего задачи в пуле подпроцессов
    Используется на КС
    """
    def __init__(self):
        self.pool = None
        self.tasks = {}
        self.log = logger.getLogger('Service')
    
    @synchronized
    def addAsyncTask(self, id, function, args):
        self.log.debug('добавление задачи id = %s в пул' % id)
        if id in self.tasks:
            # Для каждого магазина по отдельности проверяется, что задачи для него еще есть в пуле и они выполнены
            if self.tasks[id].ready():
                # если результат выполнения предыдущей операции успешный - помещаем новую задачу в пул, они создаст connection к qpid и далее далее
                if self.tasks[id].get():
                    self.log.debug('задача id = %s выполнена успешно' % id)
                    self.tasks[id] = self.pool.apply_async(function, args)
                else:
                    # если же задача по данному магазину по какой-либо причине не выполнилась - загрузка справочников по нему ПРЕКРАЩАЕТСЯ, вроде как до
                    # перезапуска nes, но это не точно
                    self.log.debug('при выполненни задачи id = %s произошла ошибка' % id)
            else:
                # если задача не выполнена, то добавлять новую смысла нет - сообщения из очереди по данному магазину и так обработаются (т.к очередь читается
                # до конца)
                self.log.debug('задача еще id = %s не обработана'% id)
        else:
            self.tasks[id] = self.pool.apply_async(function, args)
            self.log.debug('добавлена новая задача id = %s в пул' % id)
            
    def forgotBadTasks(self):
        for id in self.tasks.keys():
            if self.tasks[id].ready() and not self.tasks[id].get():
                self.log.info('remove task %s' % id)
                del self.tasks[id]

    def stopAllTasks(self):
        if self.pool:
            try:
                self.pool.terminate()
            except:
                pass
            try:
                self.pool.close()
            except:
                pass
            self.pool.join()
    
    def init(self, countOfTasks):
        self.pool = multiprocessing.Pool(countOfTasks)
