# -*- coding: UTF-8 -*-

import json
import os
import glob
import shutil


class Progress:
    def __init__(self, log, testMode=False):
        self.log = log
        self.progressPath = "/linuxcash/cash/exchangesystems/progress"
        self.nesProgressFile = os.path.join(self.progressPath, "nes.json")
        self.countProgressFile = os.path.join(self.progressPath, "counter.json")

        self.testProgressPath = "/tmp/progress"
        self.testProgressMode = testMode

        self.progressStep = 0          # Шаг обновления прогресса
        self.nextStep = 0              # Следующий шаг обновления прогресса
        self.currentObjectsCount = 0   # Объектов загружено
        self.progressEnable = False    # Использовать сохранение прогресса

    def initNesProgress(self, messageId):
        """ Инициализация сохранения прогресса """
        if not os.path.exists(self.progressPath) or not messageId:
            return

        self.progressEnable = False
        self.nextStep = 0
        self.currentObjectsCount = 0
        self.saveNesProgress(1)
        self.countProgressFile = os.path.join(self.progressPath, "counter_%s.json") % messageId

        if self.testProgressMode:
            if not os.path.exists(self.testProgressPath):
                os.makedirs(self.testProgressPath)
            shutil.copyfile(self.countProgressFile, os.path.join(self.testProgressPath, "counter.json"))

        try:
            with open(self.countProgressFile, 'r') as outfile:
                data = json.load(outfile)
                self.log.info(u'Количество объектов для загрузки %s' % data['objectsCount'])
                self.progressStep = float(data['objectsCount']) / 20
                self.nextStep = self.progressStep
                self.progressEnable = True
        except Exception as e:
            self.log.error(u'Ошибка получения кол-ва объектов из %s. %s' % (self.countProgressFile, e))

    def saveNesProgress(self, value):
        """ Сохраняет текущий прогресс в файл """
        data = {"progress": value}
        try:
            with open(self.nesProgressFile, 'w') as outfile:
                json.dump(data, outfile)
            if self.testProgressMode:
                if not os.path.exists(self.testProgressPath):
                    os.makedirs(self.testProgressPath)
                shutil.copyfile(self.nesProgressFile, os.path.join(self.testProgressPath, "nes_%s.json" % value))
        except Exception as e:
            self.log.error(u'Ошибка сохранения прогресса в %s. %s' % (self.nesProgressFile, e))

    def updateNesProgress(self):
        """ Обновление прогресса загрузки """
        if self.progressEnable:
            self.currentObjectsCount += 1
            if self.currentObjectsCount >= self.nextStep:
                self.nextStep += self.progressStep
                percent = int(5 * self.currentObjectsCount / self.progressStep)
                self.log.info(u'%d%% выполнено' % percent)
                self.saveNesProgress(percent)

    def createProgress(self, id, objectsCount):
        """ Создание файла прогресса загрузки справочников """
        if not os.path.exists(self.progressPath):
            return

        if id:
            self.countProgressFile = os.path.join(self.progressPath, "counter_%s.json") % id
        else:
            self.countProgressFile = os.path.join(self.progressPath, "counter.json")

        try:
            with open(self.countProgressFile, 'w') as progressFile:
                json.dump({'objectsCount': objectsCount}, progressFile)
        except Exception as e:
            self.log.info('Ошибка создания файла прогресса: %s' % str(e))

    def removeProgresses(self, moveBeforeDelete=False):
        """ Удаление файлов прогресса загрузки справочников """
        if moveBeforeDelete and os.path.exists(self.countProgressFile):
            if not os.path.exists(self.testProgressPath):
                os.makedirs(self.testProgressPath)
            shutil.copyfile(self.countProgressFile, os.path.join(self.testProgressPath, "counter.json"))

        try:
            if os.path.exists(self.nesProgressFile):
                os.remove(self.nesProgressFile)
        except Exception as e:
            self.log.error(u'Ошибка удаления %s. %s' % (self.nesProgressFile, e))

        # удаление всех файлов прогрессов по маске
        for file in glob.glob(os.path.join(self.progressPath, "counter*.json")):
            try:
                os.remove(file)
            except Exception as e:
                self.log.error(u'Ошибка удаления %s. %s' % (file, e))
